#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
import shutil
import subprocess
import tarfile
from pathlib import Path

import pytest


@pytest.fixture
def stub_calibration_area() -> Path:
    """
    Fixture that creates a temporary mimic of a calibration run's working area

    :return: Path to the base of the working area
    """
    path_to_lustre = Path("/lustre/aoc/cluster/pipeline/docker/workspaces")
    # Populate spool dir
    workflow_dir = "tmp0123456"
    workflow_path = path_to_lustre / "spool" / workflow_dir
    spool_dirs = ["products", "rawdata", "working"]
    spool_files = ["metadata.json", "PPR.xml"]
    for d in spool_dirs:
        (workflow_path / d).mkdir(parents=True)
    for f in spool_files:
        (workflow_path / f).touch()

    # Populate spool/products
    products_path = workflow_path / "products"
    products = [
        "16B-069_sb32814386_1_001.57685.66193635417.ms.calapply.txt",
        "16B-069_sb32814386_1_001.57685.66193635417.ms.flagversions.tgz",
        "casa_commands.log",
        "casa_pipescript.py",
        "oussid.3C286_bp.X_band.cont.I.alpha.error.fits",
        "oussid.3C286_bp.X_band.cont.I.alpha.fits",
        "oussid.3C286_bp.X_band.cont.I.mask.fits",
        "unknown.hifv_cal.auxproducts.tgz",
        "unknown.hifv_cal.pprequest.xml",
        "unknown.session_1.caltables.tgz",
        "weblog.tgz",
    ]
    for f in products:
        (products_path / f).touch()

    yield path_to_lustre

    shutil.rmtree(path_to_lustre / "spool" / workflow_dir)
    shutil.rmtree(path_to_lustre / "staging" / workflow_dir)
    shutil.rmtree(path_to_lustre / "storage" / workflow_dir)

    assert not (path_to_lustre / "spool" / workflow_dir).exists()
    assert not (path_to_lustre / "staging" / workflow_dir).exists()
    assert not (path_to_lustre / "storage" / workflow_dir).exists()


def test_calibration_table_collector(stub_calibration_area: Path):
    """
    Test that `calibration-table-collector.sh` does indeed collect the calibration tables and place them in the right
    places in the right forms
    """
    subprocess.run(["ingest_requirements/calibration-table-collector.sh", "tmp0123456", "test_sdm_id", "cal_date_time"])
    # Ensure tar exists in staging dir
    assert (stub_calibration_area / "staging" / "tmp0123456" / "test_sdm_id.cal_date_time.caltables.tar").exists()
    # Ensure tar contains correct products
    expected_products = [
        "test_sdm_id.cal_date_time.caltables.tar",
        "16B-069_sb32814386_1_001.57685.66193635417.ms.calapply.txt",
        "16B-069_sb32814386_1_001.57685.66193635417.ms.flagversions.tgz",
        "casa_commands.log",
        "casa_pipescript.py",
        "unknown.hifv_cal.auxproducts.tgz",
        "unknown.hifv_cal.pprequest.xml",
        "unknown.session_1.caltables.tgz",
        "weblog.tgz",
    ]
    with tarfile.open(
        stub_calibration_area / "staging" / "tmp0123456" / "test_sdm_id.cal_date_time.caltables.tar"
    ) as products:
        products.extractall()
        for f in (stub_calibration_area / "staging" / "tmp0123456").iterdir():
            assert f.name in expected_products
    # Ensure weblog exists in staging dir
    assert (stub_calibration_area / "staging" / "tmp0123456" / "weblog.tgz").exists()
    # Ensure tar exists in storage dir
    assert (stub_calibration_area / "storage" / "tmp0123456" / "test_sdm_id.cal_date_time.caltables.tar").exists()
